#version 150
precision highp float; precision highp int;
//Ethan Alexander Shulman 2024 "Leaves of Fall" November 14, 2024

in vec3 in_Position;
out vec4 uv;
out float time;

layout(std140) uniform ShaderviewBlock {
	vec4 mouse;
	float screenX,screenY;
	int frame,framerate;
};

#define AA_SAMPLES 16
#define ANIMATION 1

#define PI 3.141592653589793


mat2 r2d(float a) {
	float c = cos(a), s = sin(a);
	return mat2(c,s,-s,c);
}
//r2 hash http://extremelearning.com.au/unreasonable-effectiveness-of-quasirandom-sequences/
vec2 quasiRandom(float n) {
	#define R2V 1.32471795724474602596
	return fract(.5+n*(1./vec2(R2V,R2V*R2V)));
}

float twave(float x) {
	return abs(fract(x)-.5)*2.;
}

#define ITER 40
#define STEP_SZ .1

#define LEAF_SZ .05
#define LEAF_PAD 2.

#define AO_SIZE .02
#define AO_STRENGTH .5
#define AO_BIAS .2


uniform sampler2D Buffer0;

void vert() {
	#ifdef ANIMATION
	time = float(frame)/float(framerate*AA_SAMPLES);
	#else
	//screenshot 1
	time = 5.+float(frame)*.2/float(AA_SAMPLES);
	
	//screenshot 2
	//time = 36.+float(frame)*.14/float(AA_SAMPLES);
	#endif
	
	uv = vec4((in_Position.xy*vec2(screenX/screenY,1)+(quasiRandom(float(frame%AA_SAMPLES))-.5)*2./screenY),
		(in_Position.xy*.5+.5)*vec2(screenX,screenY));
	gl_Position = vec4(in_Position,1);
}
void frag() {
	vec3 rp = vec3(0,0,time*.4+cos(time*.18+1.2)),
		rd = normalize(vec3(uv.xy,1));
	vec4 c = vec4(0);
	float ao = 1.;
	
	#ifndef ANIMATION
	if (frame >= AA_SAMPLES) {
		discard;return;
	}
	#endif
		
	for (int i = 0; i < ITER; i++) {
		vec3 tp = rp;
		tp.xy *= r2d(tp.z);
		tp += cos(rp.yzx*.4);
		tp += sin(rp.zxy*9.)*.04;
		
		vec2 rep = mod(tp.xy, LEAF_SZ*LEAF_PAD*2.)-LEAF_SZ*LEAF_PAD;
		float ang = atan(rep.x,rep.y),
			thick = LEAF_SZ*.5+(cos(dot(rp,vec3(1,2,4)))*.5+.5)*LEAF_SZ*.25,
			d = length(rep)-thick-twave((ang/PI+1.)*5.+rp.z*3.8763)*(LEAF_SZ-thick)*(cos(ang*3.+rp.z*.4732)*.5+.5),
			alpha = clamp(1.-d*100., 0.,1.);
		c += vec4(vec3(.7,.35,.05)+cos(tp.xyz*vec3(9.3,12.6,15))*vec3(.2,.3,.03), 1)*alpha;
		if (c.w >= 1.) break;
		
		ao = min(1., ao+STEP_SZ*AO_BIAS);
		ao *= mix(1., clamp(d*(1./AO_SIZE),0.,1.), AO_STRENGTH);
		rp += rd*STEP_SZ;
	}
	if (c.w > 1.) c /= c.w;
	c.xyz *= ao;
	if (c.w < 1.) c.xyz += (1.-c.w)*vec3(0.515, 0.130, 0.055);//background
	
	vec4 sum = texelFetch(Buffer0,ivec2(uv.zw),0);
	#ifdef ANIMATION
	if (frame%AA_SAMPLES == 0) sum = vec4(0);
	#else
	if (frame == 0) sum = vec4(0);
	#endif
	sum += vec4(c.xyz, 1.);
	gl_FragColor = sum;
}
